@@ -35,9 +35,9 @@ const size_t CMD_RESTART_LEN     = sizeof(CMD_RESTART);
 const size_t CMD_RTP_LEN         = sizeof(CMD_RTP);
 
 const char UDPXY_COPYRIGHT_NOTICE[] =
-    "udpxy and udpxrec are Copyright (C) 2008-2011 Pavel V. Cherenkov and licensed under GNU GPLv3";
+    "udpxy and udpxrec are Copyright (C) 2008-2013 Pavel V. Cherenkov and licensed under GNU GPLv3";
 const char UDPXY_CONTACT[] =
-    "Contact by email: pcherenkov@gmail.com; ICQ # 7197767";
+    "Contact: www.udpxy.com/forum; support@udpxy.com";
 
 #ifndef TRACE_MODULE
   const char COMPILE_MODE[] = "lean";
@@ -45,10 +45,10 @@ const char UDPXY_CONTACT[] =
   const char COMPILE_MODE[] = "standard";
 #endif
 
-const char   g_udpxy_app[]        = "udpxy";
+const char   g_udpxy_app[]  = "udpxy";
 
 #ifdef UDPXREC_MOD
-const char   g_udpxrec_app[]      = "udpxrec";
+const char   g_udpxrec_app[] = "udpxrec";
 #endif
 
 const char IPv4_ALL[] = "0.0.0.0";
